require "prefabutil"

local assets=
{
	Asset("ANIM", "anim/safechest.zip"),
	Asset("ANIM", "anim/ui_safechest_3x3.zip"),
	Asset("MINIMAP_IMAGE", "safechest"),
}

local prefabs =
{
	"collapse_small",
}

local function onopen(inst) 
	inst.AnimState:PlayAnimation("open") 
	inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_open")
end 

local function onclose(inst)
	inst.AnimState:PlayAnimation("close") 
	inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_close")
end 

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	if inst.components.container then inst.components.container:DropEverything() end
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")	
	inst:Remove()
end

local function onhit(inst, worker)
	inst.AnimState:PlayAnimation("hit")
	inst.AnimState:PushAnimation("closed", false)
	if inst.components.container then 
		inst.components.container:DropEverything() 
		inst.components.container:Close()
	end
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("closed", false)
end

local slotpos = {}

for y = 2, 0, -1 do
	for x = 0, 2 do
		table.insert(slotpos, Vector3(80*x-80*2+80, 80*y-80*2+80,0))
	end
end

	
local fn = function(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon("safechest.tex")
	
	local s = 1
	inst.Transform:SetScale(s,s,s)

	inst:AddTag("structure")
	--inst:AddTag("chest")
	inst.AnimState:SetBank("safechest")
	inst.AnimState:SetBuild("safechest")
	inst.AnimState:PlayAnimation("closed", true)
	
	inst:AddComponent("inspectable")
	inst:AddComponent("container")
	inst.components.container:SetNumSlots(#slotpos)
	
	inst.components.container.onopenfn = onopen
	inst.components.container.onclosefn = onclose
	
	inst.components.container.widgetslotpos = slotpos
	inst.components.container.widgetanimbank = "ui_safechest_3x3"
	inst.components.container.widgetanimbuild = "ui_safechest_3x3"
	inst.components.container.widgetpos = Vector3(0,200,0)
	inst.components.container.side_align_tip = 160
	
	inst:AddComponent("lootdropper")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(20)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit) 
	
	local old_Destroy = inst.components.workable.Destroy
	function inst.components.workable:Destroy(destroyer)
		if destroyer.components.playercontroller == nil then
			return
		end
		return old_Destroy(self,destroyer)
	end
	
	inst:ListenForEvent( "onbuilt", onbuilt)
	MakeSnowCovered(inst, .01)	

	return inst
end

return Prefab( "common/safechest", fn, assets),
		MakePlacer("common/safechest_placer", "safechest", "safechest", "closed") 